@Resourcebrowser = do ->
  thumbnailViewButtonSelector: '.editing-button-view'
  options: {}
  filters: {}

  # The resource browser supports operations like +delete+ and +edit+ of resources that require a page
  # reload, so that all references of the resources get updated.
  reload: false

  _getBatchSize: ->
    @batchSize ||= 20

  _getThumbnailSize: ->
    @thumbnailSize ||= 'normal'

  _setThumbnailSize: (value) ->
    @thumbnailSize = value

  _appendFilterItemTo: (list, options) ->
    title = options.title || 'missing title'
    icon = options.icon || 'scrivito-resourcebrowser-icon-generic'
    query = options.query
    classname = options.classname || 'filter'

    icon = icon.replace 'editing-icon', 'scrivito-resourcebrowser-icon' # backwards compatibility

    @_filterTemplate(title, icon, query)
      .addClass(classname)
      .appendTo(list)

  _appendSeparatorTo: (list) ->
    $('<li></li>')
      .addClass('separator')
      .appendTo(list)

  _filterListTemplate: (filters) ->
    list = $('<ul></ul>')
      .addClass('scrivito-resourcebrowser-filter-items')

    @_appendFilterItemTo list,
      title: 'Selected <span class="scrivito-resourcebrowser-counter selected-total"></span>',
      icon: 'scrivito-resourcebrowser-icon-ok',
      classname: 'selected-filter'

    @_appendSeparatorTo list

    for name, options of filters
      @_appendFilterItemTo list, options

    list

  _loadFilter: () ->
    filters = @_selectedFilters()
    wrapper = @modal.find('.scrivito-resourcebrowser-filter')

    @_filterListTemplate(filters)
      .appendTo(wrapper)

  _activateInitialFilter: () ->
    @_defaultFilter().trigger('click')

  _selectedFilters: ->
    availableFilters = @filters
    selectedFilters = @options.filters
    filters = availableFilters

    if selectedFilters?
      unless $.isArray(selectedFilters)
        selectedFilters = new Array(selectedFilters)

      filters = {}

      for filterId, filter of availableFilters
        if filterId in selectedFilters
          filters[filterId] = filter

    filters

  _defaultFilter: ->
    @_filterItems().first()

  _getFilterQuery: (filter) ->
    filter.data('query')

  _defaultQuery: ->
    @_getFilterQuery(@_defaultFilter())

  _activeQuery: ->
    filter = @_filterItems().filter('.active')
    @_getFilterQuery(filter)

  _filterItems: ->
    @modal.find('ul.scrivito-resourcebrowser-filter-items li')

  _deactivateAllFilter: ->
    @_getSearch().val('')
    @_filterItems().removeClass('active')

  _triggerFilter: (filter) ->
    @_deactivateAllFilter()

    filter.addClass('active')

    query = @_getFilterQuery(filter)
    @_renderPlaceholder(query)

  _triggerSelectedFilter: (filter) ->
    if @selected.length > 0
      query = @_prepareQuery(scrivito.obj_where('id', 'equals', @selected))
      filter.data('query', query)
    else
      filter.removeData('query')

    @_triggerFilter(filter)

  _filterTemplate: (title, icon, query) ->
    filter = $('<li></li>')

    if query?
      query = @_prepareQuery(query)
      filter.data('query', query)

    icon = $('<span></span>')
      .addClass('scrivito-resourcebrowser-icon')
      .addClass(icon)
      .appendTo(filter)

    title = $('<span></span>')
      .addClass('scrivito-resourcebrowser-filter-name')
      .html(title)
      .appendTo(filter)

    filter

  _prepareQuery: (query) ->
    query.clone()
      .order('_last_changed')
      .reverse_order()
      .format('resourcebrowser')

  _save: ->
    (@options.onSave || $.noop)(@selected)

    @close()

  _delete: ->
    (@options.onDelete || $.noop)(@selected)

    $.each @selected, (index, id) =>
      item = @_getItemContainer(id)
      @_itemLoading(item)

      scrivito.delete_obj(id).then =>
        @modal.trigger('resource_change.resourcebrowser')
        item.remove()

    @_deselectAllItems()

  _getItemId: (item) ->
    $(item).closest('li.resourcebrowser-item').data('id')

  _getItemContainer: (id) ->
    $('li.resourcebrowser-item').filter ->
      id == $(this).data('id')

  _selectItem: (item) ->
    if (item.hasClass('active'))
      @_removeItem(item)
    else
      if @options.selectionMode == 'single'
        @_deselectAllItems()

      @_addItem(item)

  _addItem: (item) ->
    @_activateItem(item)

    id = @_getItemId(item)

    @selected.push(id)
    @_setSelected($.unique(@selected))

  _removeItem: (item) ->
    @_deactivateItem(item)

    selected = @selected.filter (id) =>
      id != @_getItemId(item)

    @_setSelected(selected)

  _activateItem: (item) ->
    $(item).addClass('active')

  _deactivateItem: (item) ->
    $(item).removeClass('active')

  _changeSelectedTotal: ->
    @modal.find('.selected-total').html(@selected.length)

  _setSelected: (value) ->
    @selected = value || @options.selection || []
    @_changeSelectedTotal()

  _deselectAllItems: ->
    @_setSelected([])
    @modal.find('li.resourcebrowser-item .select-item.active').removeClass('active')

  _getItems: ->
    @modal.find('.scrivito-resourcebrowser-items')

  _getContainer: ->
    @modal.find('.scrivito-resourcebrowser-thumbnails')

  _itemsPlaceholder: (count) ->
    size = @_getThumbnailSize()

    list = $('<ul></ul>')
      .addClass('items scrivito-resourcebrowser-thumbnails')
      .addClass(size)

    content = for index in [0...count] by 1
      itemTemplate = @_itemPlaceholderTemplate(index)
      list.append(itemTemplate)

    @_getItems().html(content)

  _itemPlaceholderTemplate: (index) ->
    item = $('<li></li>')
      .addClass('resourcebrowser-item')
      .attr('data-index', index)

    @_itemLoading(item)

  _itemLoading: (item) ->
    loading = @_loadingTemplate()
    $(item).html(loading)

  _renderPlaceholder: (query) ->
    @_getItems()
      .empty()

    if query?
      @_getItems().html(@_loadingTemplate())

      query
        .batch_size(@_getBatchSize())

      query.size()
        .done (total) =>
          if total > 0
            @_itemsPlaceholder(total)
            @_renderItems(query)
          else
            @_getItems().empty()

  _renderItems: (query, index = 0) ->
    query.load_batch()
      .done (result, next) =>
        objects = result.hits
        @_replacePlaceholder(objects, index)

        if next?
          start = index + objects.length
          @_renderItems(next, start)

  _replacePlaceholder: (objects, startIndex) ->
    $(objects).each (index, object) =>
      elementIndex = startIndex + index
      template = @_itemTemplate(object)

      @modal.find("li.resourcebrowser-item[data-index=#{elementIndex}]")
        .html(template)
        .data('id', object.id)

  _itemTemplate: (object) ->
    url = object.preview
    title = object.title || object.id
    id = object.id

    wrapper = $('<div></div>')
      .addClass('scrivito-resourcebrowser-item-wrapper')

    preview = $('<div></div>')
      .addClass('scrivito-resourcebrowser-preview')
      .appendTo(wrapper)

    inspect = $('<span></span>')
      .addClass('scrivito-resourcebrowser-inspect')
      .appendTo(wrapper)

    image = if url?
      $('<img />')
        .attr('src', url)
    else
      $('<span></span>')
        .addClass('scrivito-resourcebrowser-icon')
        .addClass('scrivito-resourcebrowser-icon-generic')

    image.appendTo(preview)

    meta = $('<div></div>')
      .addClass('scrivito-resourcebrowser-meta')
      .appendTo(wrapper)

    title = $('<span></span>')
      .addClass('scrivito-resourcebrowser-thumbnails-name')
      .html(title)
      .appendTo(meta)

    select = $('<span></span>')
      .addClass('scrivito-resourcebrowser-thumbnails-select select-item')
      .appendTo(title)

    if id in @selected
      select.addClass('active')

    wrapper

  _getSearch: ->
    @modal.find('input.search-field')

  _triggerSearch: ->
    term = @_getSearch().val()
    query = @_prepareQuery(@_activeQuery())

    if term? && term.length > 0
      query.and('*', 'contains_prefix', term)

    @_renderPlaceholder(query)

  _initializeBindings: ->
    $(window).resize =>
      @_center(@modal)

    @modal.on 'keyup', 'input.search-field', (event) =>
      if event.keyCode == 13
        @_triggerSearch()

    @modal.on 'click', 'button.search-field-button', (event) =>
      event.preventDefault()
      @_triggerSearch()

    @modal.on 'click', 'li.resourcebrowser-item', (event) =>
      unless $(event.target).hasClass('scrivito-resourcebrowser-inspect')
        item = $(event.currentTarget).find('.select-item')
        @_selectItem(item)

    @modal.on 'click', '.resourcebrowser-save', (event) =>
      event.preventDefault()
      @_save()

    @modal.on 'click', '.resourcebrowser-delete', (event) =>
      event.preventDefault()
      @_delete()

    @modal.on 'click', '.resourcebrowser-close', (event) =>
      event.preventDefault()
      @close()

    @modal.on 'click', 'li.filter', (event) =>
      event.preventDefault()
      @_triggerFilter($(event.currentTarget))

    @modal.on 'click', 'li.selected-filter', (event) =>
      event.preventDefault()
      @_triggerSelectedFilter($(event.currentTarget))

    @modal.on 'click', @thumbnailViewButtonSelector, (event) =>
      size = $(event.currentTarget).data('size')
      @_changeThumbnailSize(size)

    @modal.on 'resource_change.resourcebrowser', (event) =>
      @reload = true

    $(document).on 'keyup.resourcebrowser', (event) =>
      event.stopImmediatePropagation()

      if event.keyCode == 27
        # Make sure to remove the event handler after
        # +stopImmediatePropagation()+, otherwise all ESC keys are caught and
        # not propagated.
        $(document).off 'keyup.resourcebrowser'

        @close()

  _loadModal: () ->
    @overlay = $('<div></div>')
      .addClass('editing-overlay show')
      .appendTo($('body'))

    innerHtml = """
        <div class="scrivito-resourcebrowser-body">
          <div class="scrivito-resourcebrowser-wrapper">
            <div class="scrivito-resourcebrowser-topbar">
              <div class="scrivito-resourcebrowser-search">
                <input class="search-field" placeholder="Search", type="text" />
                <button class="search-field-button">
                  Search
                </button>
              </div>
              <span class="editing-button-view" data-size="normal-list">
                <span class="scrivito-resourcebrowser-icon scrivito-resourcebrowser-icon-list-medium"
                    title="List view"></span>
              </span>
              <span class="editing-button-view" data-size="large">
                <span class="scrivito-resourcebrowser-icon scrivito-resourcebrowser-icon-th-large"
                    title="Large thumbnails"></span>
              </span>
              <span class="editing-button-view" data-size="big">
                <span class="scrivito-resourcebrowser-icon scrivito-resourcebrowser-icon-th"
                    title="Big thumbnails"></span>
              </span>
              <span class="editing-button-view" data-size="normal">
                <span class="scrivito-resourcebrowser-icon scrivito-resourcebrowser-icon-th-medium"
                    title="Thumbnails"></span>
              </span>
              <span class="editing-button-view" data-size="small">
                <span class="scrivito-resourcebrowser-icon scrivito-resourcebrowser-icon-th-small"
                    title="Small thumbnails"></span>
              </span>
            </div>
            <div class="scrivito-resourcebrowser-filter"></div>
            <div class="scrivito-resourcebrowser-items"></div>
          </div>
          <div class="scrivito-resourcebrowser-inspector"></div>
        </div>
        <div class="scrivito-resourcebrowser-footer">
          <a class="editing-button resourcebrowser-close">
            Cancel
          </a>
          <a class="editing-button editing-green resourcebrowser-save">
            Select
            <span class="scrivito-resourcebrowser-counter selected-total"></span>
          </a>
          <a class="editing-button editing-red resourcebrowser-delete">
            Delete
            <span class="scrivito-resourcebrowser-counter selected-total"></span>
          </a>
        </div>
        """

    @modal = $('<div></div>')
      .addClass('scrivito-resourcebrowser show')
      .attr('id', 'scrivito-resourcebrowser')
      .html(innerHtml)
      .appendTo($('body'))

    @_center(@modal)

    @_initializeBindings()

  _center: (domElement) ->
    if domElement
      domElement = $(domElement)
      domElement.css(
        marginLeft: -domElement.innerWidth() / 2
        marginTop: -domElement.innerHeight() / 2
        left: '50%'
      )

  _loadingTemplate: ->
    icon = $('<i></i>')
      .addClass('scrivito-resourcebrowser-icon scrivito-resourcebrowser-icon-refresh')

    $('<div></div>')
      .addClass('scrivito-resourcebrowser-loading')
      .html(icon)

  _changeThumbnailSize: (size) ->
    size ||= @_getThumbnailSize()
    @_setThumbnailSize(size)

    transitionListener = 'webkitTransitionEnd.resourcebrowser otransitionend.resourcebrowser oTransitionEnd.resourcebrowser msTransitionEnd.resourcebrowser transitionend.resourcebrowser'
    @modal.on transitionListener, 'li.resourcebrowser-item', (event) =>
      @modal.off transitionListener

    @_getContainer()
      .removeClass('small normal big large normal-list')
      .addClass(size)

    @modal.find(@thumbnailViewButtonSelector)
      .removeClass('active')
      .filter("[data-size='#{size}']")
      .addClass('active')

  close: ->
    (@options.onClose || $.noop)()

    @overlay.remove()
    @modal.remove()

    if @reload
      $('body').trigger('infopark_reload')

  open: (options) ->
    @options = options

    @_loadModal()
    @_loadFilter()
    @_setSelected()
    @_renderPlaceholder(@_defaultQuery())
    @_changeThumbnailSize()
    @_activateInitialFilter()

    ResourcebrowserInspector.init(@modal)
    ResourcebrowserUploader.init(@modal)

    ResourcebrowserUploader.onUploadStart = (files) =>
      @upload_errors = false

    ResourcebrowserUploader.onUploadFailure = (errors) =>
      @upload_errors = errors
      # console.error('Resourcebrowser uploader errors: ', JSON.stringify(upload_errors))

    ResourcebrowserUploader.onUploadSuccess = (objs) =>
      @_renderPlaceholder(@_activeQuery()) unless @upload_errors
